//////////////////////////////////////////////////////////////////////////////////////////////
// Clase para resolver un sistema lineal mediante CGM                                       //
//                                                                                          //
// Anlisis y Diseo y Programacin:                                                        //
//                                                                                          //
// Nombre:   Antonio Carrillo Ledesma                                                       //
// E-mail:   acl@www.mmc.geofisica.unam.mx                                                  //
// Pgina:   http://www.mmc.geofisica.unam.mx/acl                                           //
//                                                                                          //
//                                                                                          //
// Este programa es software libre. Puede redistribuirlo y/o modificarlo                    //
// bajo los trminos de la Licencia Pblica General de GNU segn es                         //
// publicada por la Free Software Foundation, bien de la versin 2 de                       //
// dicha Licencia o bien (segn su eleccin) de cualquier versin                           //
// posterior.                                                                               //
//                                                                                          //
// Este programa se distribuye con la esperanza de que sea til, pero SIN                   //
// NINGUNA GARANTA, incluso sin la garanta MERCANTIL implcita o sin                      //
// garantizar la CONVENIENCIA PARA UN PROPSITO PARTICULAR. Vase la                        //
// Licencia Pblica General de GNU para ms detalles.                                       //
//                                                                                          //
// Debera haber recibido una copia de la Licencia Pblica General junto                    //
// con este programa. Si no ha sido as, escriba a la Free Software                         //
// Foundation, Inc., en 675 Mass Ave, Cambridge, MA 02139, EEUU.                            //
//                                                                                          //
//                                                                                          //
//////////////////////////////////////////////////////////////////////////////////////////////



#ifndef __BCGM__
#define __BCGM__

#include "ResuelveSistemaLineal.hpp"
#include "MultOp.hpp"
#include "ProductoPunto.hpp"


/// Clase para resolucin del sistema lineal mediante CGM
/** @author Antonio Carrillo Ledesma
    @date primavera 2009
    @version 1.0.1
    @bug No hay errores conocidos
*/
class BCGM: public ResuelveSistemaLineal
{


protected:

   // Operador
   MultOp *A;

   /// Producto Punto
   ProductoPunto *prodP;

   /// Nmero mximo de iteraciones
   int Iter;

   /// Tolerancia
   ldouble Ep;


public:

   /// Constructor de la clase
   /** @param a Puntero a la implementacin a la multiplicacin de la matriz por el vector
       @param prod Puntero a la implementacin del producto punto de dos vectores
       @param iter Mximo nmero de interaciones
       @param ep Tolerancia mnima */
   BCGM(MultOp &a, ProductoPunto &prod, int iter, ldouble ep) : ResuelveSistemaLineal()
   {
      A = &a;
      prodP = &prod;
      Iter = iter;
      Ep = ep;
   }

   /// Resuelve el sistema lineal
   void resuelve(void);


   /// Resuelve el sistema lineal
   /** @param x Puntero a un vector, solucin del sistema lineal
       @param b Puntero a un vector, lado derecho del sistema lineal */
   void resuelve(Vector *x, Vector *b)
   {
      X = x;
      B = b;
      resuelve();
   }

   /// Establece la tolerancia del metodo
   /** @param eps Tolerancia del metodo */
   void tolerancia(ldouble eps)
   {   
	Ep = eps;
   }

   /// Establece el maximo numero de iteraciones
   /** @param iter Iteraciones del metodo */
   void iteraciones(int iter)
   {   
	Iter = iter;
   }
      
   
};

/**
Esta clase implementa los componentes para la resolucin del sistema lineal mediante CGM
@example EjemploResuelveSistemaLineal.cpp */

#endif
